package util;


import javax.faces.application.FacesMessage;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.servlet.ServletContext;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.io.IOException;
import java.net.URL;
import java.text.MessageFormat;
import java.text.NumberFormat;
import java.util.*;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * @author jon_m
 * @since 06-Jun-2008 12:11:37
 *        <p/>
 *        DESCRIPTION
 *        Provides helper methods for JSF related functions
 */
public class JSFUtils {

	private static Logger log = Logger.getLogger("util.JSFUtils");

    protected static ClassLoader getCurrentClassLoader(Object defaultObject) {
        ClassLoader loader = Thread.currentThread().getContextClassLoader();
        if (loader == null) {
            loader = defaultObject.getClass().getClassLoader();
        }
        return loader;
    }


    /**
     * Gets a message from the message bundle
     *
     * @param bundleName the name of the bundle
     * @param key        message key in bundle
     * @param params     any params for message
     * @param locale     the locale for the bundle
     * @return messge string with translated params
     */
    protected static String getMessageResourceString(String bundleName, String key, Object params[], Locale locale) {
        String text = null;
        ResourceBundle bundle = ResourceBundle.getBundle(bundleName, locale, getCurrentClassLoader(bundleName));
        try {
            text = bundle.getString(key);
        } catch (MissingResourceException e) {
            text = "?? key " + key + " not found ??";
        }
        if (params != null) {
            MessageFormat mf = new MessageFormat(text, locale);
            text = mf.format(params, new StringBuffer(), null).toString();
        }
        return text;
    }

    public static String getMessage(String key, Object[] params, Locale locale) {
        log.info("getMessage: " + key);
        FacesContext context = FacesContext.getCurrentInstance();

        if (locale == null) {
            locale = FacesContext.getCurrentInstance().getViewRoot().getLocale();
        }
        try {
            return getMessageResourceString(context.getApplication().getMessageBundle(), key, params,
                    locale);
        } catch (Exception e) {
            return e.getMessage();
        }
    }

    public static String getMessage(String value, Locale locale) {
        return getMessage(value, null, locale);
    }

    public static String getCookie(String cookieKey) {
        FacesContext facesContext = FacesContext.getCurrentInstance();
        Cookie cookies[] = ((HttpServletRequest) facesContext.getExternalContext().getRequest()).getCookies();

        String cookieName;
        if (cookies != null && cookies.length > 0) {
            for (Cookie aCookie : cookies) {

                cookieName = aCookie.getName();
                if (cookieName.equals(cookieKey)) {
                    return aCookie.getValue();
                }
            }
        }
        return "";
    }

    public static void setCookie(String cookieKey, String cookieValue) {
        
            log.info("setCookie [" + cookieKey + "] = [" + cookieValue + "]");
        Cookie cookie = new Cookie(cookieKey, cookieValue);
        cookie.setMaxAge(Integer.MAX_VALUE);
        FacesContext facesContext = FacesContext.getCurrentInstance();
        ((HttpServletResponse) facesContext.getExternalContext().getResponse()).addCookie(cookie);

    }

    public static void setCookie(String path, String cookieKey, String cookieValue) {
        
            log.info("setCookie [" + cookieKey + "] = [" + cookieValue + "] path [" + path + "]");
        Cookie cookie = new Cookie(cookieKey, cookieValue);
        cookie.setMaxAge(Integer.MAX_VALUE);
        cookie.setPath(path);
        FacesContext facesContext = FacesContext.getCurrentInstance();
        ((HttpServletResponse) facesContext.getExternalContext().getResponse()).addCookie(cookie);

    }

    public static void setCookie(Cookie cookie) {
        
            log.info("setCookie (from Cookie) [" + cookie.getName() + "] = [" + cookie.getValue() + "]");

        FacesContext facesContext = FacesContext.getCurrentInstance();
        HttpServletResponse response = (HttpServletResponse) facesContext.getExternalContext().
                getResponse();
        cookie.setMaxAge(Integer.MAX_VALUE);
        response.addCookie(cookie);

    }


    public static Object getBean(String beanName) {
        ExternalContext extContext = FacesContext.getCurrentInstance().getExternalContext();
        HttpSession session = (HttpSession) extContext.getSession(false);

        if (session == null) {
            log.warning("No session found");
            return null;
        }
        HttpServletRequest request = (HttpServletRequest) extContext.getRequest();
//        dumpAttributes(request);
        Object obj = request.getAttribute(beanName);
        if (obj != null) {
            log.info("Found [" + beanName + "] in request scope. Hashcode: " + obj.hashCode());
            return obj;
        }

        obj = session.getAttribute(beanName);
//        dumpAttributes(session);
        if (obj != null) {
            log.info("Found [" + beanName + "] in session scope. Hashcode: " + obj.hashCode());
            return obj;
        }

        obj = extContext.getApplicationMap().get(beanName);
        if (obj != null) {
            log.info("Found [" + beanName + "] in application scope. Hashcode: " + obj.hashCode());
        }
        return obj;

    }

    private static void dumpAttributes(HttpServletRequest request) {
        Enumeration attributeNames = request.getAttributeNames();
        while (attributeNames.hasMoreElements()) {
            String s = (String) attributeNames.nextElement();
            log.info("Request scope [" + s + "] = [" + request.getAttribute(s) + "]");
        }
    }

    private static void dumpAttributes(HttpSession session) {
        Enumeration attributeNames = session.getAttributeNames();
        while (attributeNames.hasMoreElements()) {
            String s = (String) attributeNames.nextElement();
            log.info("Session scope [" + s + "] = [" + session.getAttribute(s) + "]");
        }
    }

    public static <T> T getBean(String beanName, Class<T> impl) {
        FacesContext context = FacesContext.getCurrentInstance();
        HttpSession session = (HttpSession) context.getExternalContext().getSession(false);

        Object attribute = getBean(beanName);
        if (attribute != null)
            return (T) attribute;

        try {
            log.info("No object with key [" + beanName + "] found in session or application. Creating new and adding to session");
            T t = impl.newInstance();
            session.setAttribute(beanName, t);
            return t;
        } catch (Exception e) {
            log.severe("Failed to create instance of [" + impl + "]: " + e.getMessage());
        }

        return null;

    }

    public static void setApplicationBean(String key, Object obj) {

        FacesContext context = FacesContext.getCurrentInstance();
        Map<String, Object> map = context.getExternalContext().getApplicationMap();
        if (!map.containsKey(key)) {
            log.info("Adding [" + key + "] to application scope. Hashcode: " + obj.hashCode());
            map.put(key, obj);
        }
    }

    public static void setSessionBean(String key, Object obj) {

        setSessionBean(key, obj, Boolean.FALSE);
    }

    public static void setSessionBean(String key, Object obj, Boolean replace) {

        FacesContext context = FacesContext.getCurrentInstance();
        if (context == null)
            return;
        
        Map<String, Object> map = context.getExternalContext().getSessionMap();
        if (replace || !map.containsKey(key)) {
            log.info("Adding [" + key + "] to session scope. Hashcode: " + obj.hashCode());
            map.put(key, obj);
        }
    }

    public static void AddExceptionMessage(Exception e, Locale locale) {
        log.log(Level.SEVERE,"Exception: " + e,e);
        
            String s = e.toString();
            FacesContext.getCurrentInstance().addMessage("", new FacesMessage(FacesMessage.SEVERITY_ERROR, s, s));
            log.info("AddExceptionMessage : " + s);
        
    }

    }
